@@ -20,6 +20,7 @@
 
 #include "KCupsRequest.h"
 
+#include "KIppRequest.h"
 #include "KCupsJob.h"
 #include "KCupsPrinter.h"
 
@@ -47,28 +48,25 @@ QString KCupsRequest::serverError() const
 {
     switch (error()) {
     case IPP_SERVICE_UNAVAILABLE:
-        return i18n("Service is unavailable");
+        return i18n("Print service is unavailable");
     case IPP_NOT_FOUND :
         return i18n("Not found");
     default : // In this case we don't want to map all enums
         kWarning() << "status unrecognised: " << error();
-        return QString();
+        return QString::fromUtf8(ippErrorString(error()));
     }
 }
 
 void KCupsRequest::getPPDS(const QString &make)
 {
     if (m_connection->readyToStart()) {
-        QVariantHash request;
-        if (!make.isEmpty()){
-            request["ppd-make-and-model"] = make;
+        KIppRequest request(CUPS_GET_PPDS, "/");
+        if (!make.isEmpty()) {
+            request.addString(IPP_TAG_PRINTER, IPP_TAG_TEXT, KCUPS_PPD_MAKE_AND_MODEL, make);
         }
-        request["need-dest-name"] = false;
 
-        m_ppds = m_connection->request(CUPS_GET_PPDS,
-                                       "/",
-                                       request,
-                                       true);
+        m_ppds = m_connection->request(request, IPP_TAG_PRINTER);
+
         setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
@@ -129,7 +127,7 @@ void KCupsRequest::getDevices(int timeout, QStringList includeSchemes, QStringLi
                            exclude,
                            (cups_device_cb_t) choose_device_cb,
                            this);
-        } while (m_connection->retry("/admin/"));
+        } while (m_connection->retry("/admin/", CUPS_GET_DEVICES));
         setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished(true);
     } else {
@@ -141,26 +139,20 @@ void KCupsRequest::getDevices(int timeout, QStringList includeSchemes, QStringLi
 // "printer-is-default" attribute BUT it does not get user
 // defined default printer, see cupsGetDefault() on www.cups.org for details
 
-void KCupsRequest::getPrinters(QStringList attributes, cups_ptype_t mask)
-{
-    QVariantHash arguments;
-    arguments[KCUPS_PRINTER_TYPE_MASK] = mask;
-    getPrinters(attributes, arguments);
-}
-
-void KCupsRequest::getPrinters(QStringList attributes, const QVariantHash &arguments)
+void KCupsRequest::getPrinters(QStringList attributes, int mask)
 {
     if (m_connection->readyToStart()) {
-        QVariantHash request = arguments;
-        request[KCUPS_PRINTER_TYPE] = CUPS_PRINTER_LOCAL;
-        request["requested-attributes"] = attributes;
-        request["need-dest-name"] = true;
+        KIppRequest request(CUPS_GET_PRINTERS, "/");
+        request.addInteger(IPP_TAG_OPERATION, IPP_TAG_ENUM, KCUPS_PRINTER_TYPE, CUPS_PRINTER_LOCAL);
+        if (!attributes.isEmpty()) {
+            request.addStringList(IPP_TAG_OPERATION, IPP_TAG_KEYWORD, KCUPS_REQUESTED_ATTRIBUTES, attributes);
+        }
+        if (mask != -1) {
+            request.addInteger(IPP_TAG_OPERATION, IPP_TAG_ENUM, KCUPS_PRINTER_TYPE_MASK, mask);
+        }
 
         ReturnArguments ret;
-        ret = m_connection->request(CUPS_GET_PRINTERS,
-                                    "/",
-                                    request,
-                                    true);
+        ret = m_connection->request(request, IPP_TAG_PRINTER);
 
         foreach (const QVariantHash &arguments, ret) {
             m_printers << KCupsPrinter(arguments);
@@ -169,24 +161,21 @@ void KCupsRequest::getPrinters(QStringList attributes, const QVariantHash &argum
         setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
-        invokeMethod("getPrinters", qVariantFromValue(attributes), arguments);
+        invokeMethod("getPrinters", qVariantFromValue(attributes), mask);
     }
 }
 
 void KCupsRequest::getPrinterAttributes(const QString &printerName, bool isClass, QStringList attributes)
 {
     if (m_connection->readyToStart()) {
-        QVariantHash request;
-        request[KCUPS_PRINTER_NAME] = printerName;
-        request["printer-is-class"] = isClass;
-        request["need-dest-name"] = false; // we don't need a dest name since it's a single item
-        request["requested-attributes"] = attributes;
+        KIppRequest request(IPP_GET_PRINTER_ATTRIBUTES, "/admin/");
+
+        request.addPrinterUri(printerName, isClass);
+        request.addInteger(IPP_TAG_OPERATION, IPP_TAG_ENUM, KCUPS_PRINTER_TYPE, CUPS_PRINTER_LOCAL);
+        request.addStringList(IPP_TAG_OPERATION, IPP_TAG_KEYWORD, KCUPS_REQUESTED_ATTRIBUTES, attributes);
 
         ReturnArguments ret;
-        ret = m_connection->request(IPP_GET_PRINTER_ATTRIBUTES,
-                                    "/admin/",
-                                    request,
-                                    true);
+        ret = m_connection->request(request, IPP_TAG_PRINTER);
 
         foreach (const QVariantHash &arguments, ret) {
             // Inject the printer name back to the arguments hash
@@ -205,30 +194,24 @@ void KCupsRequest::getPrinterAttributes(const QString &printerName, bool isClass
 void KCupsRequest::getJobs(const QString &printerName, bool myJobs, int whichJobs, QStringList attributes)
 {
     if (m_connection->readyToStart()) {
-        QVariantHash request;
-        // printer-uri makes the Name of the Job and owner came blank lol
-        request[KCUPS_PRINTER_NAME] = printerName;
+        KIppRequest request(IPP_GET_JOBS, "/");
 
-        if (myJobs) {
-            request["my-jobs"] = myJobs;
-        }
+        // printer-uri makes the Name of the Job and owner came blank lol
+        request.addPrinterUri(printerName, false);
+        request.addInteger(IPP_TAG_OPERATION, IPP_TAG_ENUM, KCUPS_PRINTER_TYPE, CUPS_PRINTER_LOCAL);
+        request.addStringList(IPP_TAG_OPERATION, IPP_TAG_KEYWORD, KCUPS_REQUESTED_ATTRIBUTES, attributes);
+
+        request.addInteger(IPP_TAG_OPERATION, IPP_TAG_ENUM, KCUPS_MY_JOBS, myJobs);
 
         if (whichJobs == CUPS_WHICHJOBS_COMPLETED) {
-            request["which-jobs"] = "completed";
+            request.addString(IPP_TAG_OPERATION, IPP_TAG_KEYWORD, KCUPS_WHICH_JOBS, "completed");
         } else if (whichJobs == CUPS_WHICHJOBS_ALL) {
-            request["which-jobs"] = "all";
+            request.addString(IPP_TAG_OPERATION, IPP_TAG_KEYWORD, KCUPS_WHICH_JOBS, "all");
         }
 
-        if (!attributes.isEmpty()) {
-            request["requested-attributes"] = attributes;
-        }
-        request["group-tag-qt"] = IPP_TAG_JOB;
-
         ReturnArguments ret;
-        ret = m_connection->request(IPP_GET_JOBS,
-                                    "/",
-                                    request,
-                                    true);
+        ret = m_connection->request(request, IPP_TAG_JOB);
+
         foreach (const QVariantHash &arguments, ret) {
             m_jobs << KCupsJob(arguments);
         }
@@ -243,17 +226,16 @@ void KCupsRequest::getJobs(const QString &printerName, bool myJobs, int whichJob
 void KCupsRequest::getJobAttributes(int jobId, const QString &printerUri, QStringList attributes)
 {
     if (m_connection->readyToStart()) {
-        QVariantHash request;
-        request[KCUPS_JOB_ID] = jobId;
-        request[KCUPS_PRINTER_URI] = printerUri;
-        request["need-dest-name"] = false; // we don't need a dest name since it's a single list
-        request["requested-attributes"] = attributes;
+        KIppRequest request(IPP_GET_JOB_ATTRIBUTES, "/admin/");
+
+        request.addString(IPP_TAG_OPERATION, IPP_TAG_URI, KCUPS_PRINTER_URI, printerUri);
+        request.addInteger(IPP_TAG_OPERATION, IPP_TAG_ENUM, KCUPS_PRINTER_TYPE, CUPS_PRINTER_LOCAL);
+        request.addStringList(IPP_TAG_OPERATION, IPP_TAG_KEYWORD, KCUPS_REQUESTED_ATTRIBUTES, attributes);
+
+        request.addInteger(IPP_TAG_OPERATION, IPP_TAG_INTEGER, KCUPS_JOB_ID, jobId);
 
         ReturnArguments ret;
-        ret = m_connection->request(IPP_GET_JOB_ATTRIBUTES,
-                                    "/admin/",
-                                    request,
-                                    true);
+        ret = m_connection->request(request, IPP_TAG_PRINTER);
 
         foreach (const QVariantHash &arguments, ret) {
             m_jobs << KCupsJob(arguments);
@@ -287,7 +269,7 @@ void KCupsRequest::getServerSettings()
             }
 
             m_server = KCupsServer(arguments);
-        } while (m_connection->retry("/admin/"));
+        } while (m_connection->retry("/admin/", -1));
         setFinished();
     } else {
         invokeMethod("getServerSettings");
@@ -303,7 +285,7 @@ void KCupsRequest::getPrinterPPD(const QString &printerName)
             kDebug() << filename;
             m_ppdFile = filename;
             kDebug() << m_ppdFile;
-        } while (m_connection->retry("/"));
+        } while (m_connection->retry("/", CUPS_GET_PPD));
         setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
@@ -330,7 +312,7 @@ void KCupsRequest::setServerSettings(const KCupsServer &server)
 
             cupsAdminSetServerSettings(CUPS_HTTP_DEFAULT, num_settings, settings);
             cupsFreeOptions(num_settings, settings);
-        } while (m_connection->retry("/admin/"));
+        } while (m_connection->retry("/admin/", -1));
         setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
@@ -340,87 +322,81 @@ void KCupsRequest::setServerSettings(const KCupsServer &server)
 
 void KCupsRequest::addOrModifyPrinter(const QString &printerName, const QVariantHash &attributes, const QString &filename)
 {
-    QVariantHash request = attributes;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    request["printer-is-class"] = false;
-    if (!filename.isEmpty()) {
-        request["filename"] = filename;
-    }
+    KIppRequest request(CUPS_ADD_MODIFY_PRINTER, "/admin/", filename);
+    request.addPrinterUri(printerName);
+    request.addVariantValues(attributes);
 
-    doOperation(CUPS_ADD_MODIFY_PRINTER, QLatin1String("/admin/"), request);
+    process(request);
 }
 
 void KCupsRequest::addOrModifyClass(const QString &printerName, const QVariantHash &attributes)
 {
-    QVariantHash request = attributes;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    request["printer-is-class"] = true;
+    KIppRequest request(CUPS_ADD_MODIFY_CLASS, "/admin/");
+    request.addPrinterUri(printerName, true);
+    request.addVariantValues(attributes);
 
-    doOperation(CUPS_ADD_MODIFY_CLASS, QLatin1String("/admin/"), request);
+    process(request);
 }
 
 void KCupsRequest::setShared(const QString &printerName, bool isClass, bool shared)
 {
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    request["printer-is-class"] = isClass;
-    request[KCUPS_PRINTER_IS_SHARED] = shared;
-    request["need-dest-name"] = true;
+    KIppRequest request(isClass ? CUPS_ADD_MODIFY_CLASS : CUPS_ADD_MODIFY_PRINTER, "/admin/");
+    request.addPrinterUri(printerName, isClass);
+    request.addBoolean(IPP_TAG_OPERATION, KCUPS_PRINTER_IS_SHARED, shared);
 
-    ipp_op_e operation;
-    operation = isClass ? CUPS_ADD_MODIFY_CLASS : CUPS_ADD_MODIFY_PRINTER;
-
-    doOperation(operation, QLatin1String("/admin/"), request);
+    process(request);
 }
 
 void KCupsRequest::pausePrinter(const QString &printerName)
 {
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    doOperation(IPP_PAUSE_PRINTER, QLatin1String("/admin/"), request);
+    KIppRequest request(IPP_PAUSE_PRINTER, "/admin/");
+    request.addPrinterUri(printerName);
+
+    process(request);
 }
 
 void KCupsRequest::resumePrinter(const QString &printerName)
 {
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    doOperation(IPP_RESUME_PRINTER, QLatin1String("/admin/"), request);
+    KIppRequest request(IPP_RESUME_PRINTER, "/admin/");
+    request.addPrinterUri(printerName);
+
+    process(request);
 }
 
-void KCupsRequest::rejectJobs(const QString &printer)
+void KCupsRequest::rejectJobs(const QString &printerName)
 {
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = printer;
-    doOperation(CUPS_REJECT_JOBS, QLatin1String("/admin/"), request);
+    KIppRequest request(CUPS_REJECT_JOBS, "/admin/");
+    request.addPrinterUri(printerName);
+
+    process(request);
 }
 
 void KCupsRequest::acceptJobs(const QString &printerName)
 {
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    doOperation(CUPS_ACCEPT_JOBS, QLatin1String("/admin/"), request);
+    KIppRequest request(CUPS_ACCEPT_JOBS, "/admin/");
+    request.addPrinterUri(printerName);
+
+    process(request);
 }
 
 void KCupsRequest::setDefaultPrinter(const QString &printerName)
 {
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    doOperation(CUPS_SET_DEFAULT, QLatin1String("/admin/"), request);
+    KIppRequest request(CUPS_SET_DEFAULT, "/admin/");
+    request.addPrinterUri(printerName);
+
+    process(request);
 }
 
 void KCupsRequest::deletePrinter(const QString &printerName)
 {
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    doOperation(CUPS_DELETE_PRINTER, QLatin1String("/admin/"), request);
+    KIppRequest request(CUPS_DELETE_PRINTER, "/admin/");
+    request.addPrinterUri(printerName);
+
+    process(request);
 }
 
 void KCupsRequest::printTestPage(const QString &printerName, bool isClass)
 {
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    request["printer-is-class"] = isClass;
-    request[KCUPS_JOB_NAME] = i18n("Test Page");
     QString resource; /* POST resource path */
     QString filename; /* Test page filename */
     QString datadir;  /* CUPS_DATADIR env var */
@@ -433,7 +409,6 @@ void KCupsRequest::printTestPage(const QString &printerName, bool isClass)
         datadir = CUPS_DATADIR;
     }
     filename = datadir % QLatin1String("/data/testprint");
-    request["filename"] = filename;
 
     /*
      * Point to the printer/class...
@@ -444,7 +419,11 @@ void KCupsRequest::printTestPage(const QString &printerName, bool isClass)
         resource = QLatin1String("/printers/") % printerName;
     }
 
-    doOperation(IPP_PRINT_JOB, resource, request);
+    KIppRequest request(IPP_PRINT_JOB, resource.toUtf8(), filename);
+    request.addPrinterUri(printerName);
+    request.addString(IPP_TAG_OPERATION, IPP_TAG_NAME, KCUPS_JOB_NAME, i18n("Test Page"));
+
+    process(request);
 }
 
 void KCupsRequest::printCommand(const QString &printerName, const QString &command, const QString &title)
@@ -502,7 +481,7 @@ void KCupsRequest::printCommand(const QString &printerName, const QString &comma
                 setFinished();
                 return; // Return to avoid a new try
             }
-        } while (m_connection->retry("/"));
+        } while (m_connection->retry("/", IPP_CREATE_JOB));
         setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
@@ -512,38 +491,38 @@ void KCupsRequest::printCommand(const QString &printerName, const QString &comma
 
 void KCupsRequest::cancelJob(const QString &printerName, int jobId)
 {
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    request[KCUPS_JOB_ID] = jobId;
+    KIppRequest request(IPP_CANCEL_JOB, "/jobs/");
+    request.addPrinterUri(printerName);
+    request.addInteger(IPP_TAG_OPERATION, IPP_TAG_INTEGER, KCUPS_JOB_ID, jobId);
 
-    doOperation(IPP_CANCEL_JOB, QLatin1String("/jobs/"), request);
+    process(request);
 }
 
 void KCupsRequest::holdJob(const QString &printerName, int jobId)
 {
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    request[KCUPS_JOB_ID] = jobId;
+    KIppRequest request(IPP_HOLD_JOB, "/jobs/");
+    request.addPrinterUri(printerName);
+    request.addInteger(IPP_TAG_OPERATION, IPP_TAG_INTEGER, KCUPS_JOB_ID, jobId);
 
-    doOperation(IPP_HOLD_JOB, QLatin1String("/jobs/"), request);
+    process(request);
 }
 
 void KCupsRequest::releaseJob(const QString &printerName, int jobId)
 {
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    request[KCUPS_JOB_ID] = jobId;
+    KIppRequest request(IPP_RELEASE_JOB, "/jobs/");
+    request.addPrinterUri(printerName);
+    request.addInteger(IPP_TAG_OPERATION, IPP_TAG_INTEGER, KCUPS_JOB_ID, jobId);
 
-    doOperation(IPP_RELEASE_JOB, QLatin1String("/jobs/"), request);
+    process(request);
 }
 
 void KCupsRequest::restartJob(const QString &printerName, int jobId)
 {
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = printerName;
-    request[KCUPS_JOB_ID] = jobId;
+    KIppRequest request(IPP_RESTART_JOB, "/jobs/");
+    request.addPrinterUri(printerName);
+    request.addInteger(IPP_TAG_OPERATION, IPP_TAG_INTEGER, KCUPS_JOB_ID, jobId);
 
-    doOperation(IPP_RESTART_JOB, QLatin1String("/jobs/"), request);
+    process(request);
 }
 
 void KCupsRequest::moveJob(const QString &fromPrinterName, int jobId, const QString &toPrinterName)
@@ -554,12 +533,12 @@ void KCupsRequest::moveJob(const QString &fromPrinterName, int jobId, const QStr
         return;
     }
 
-    QVariantHash request;
-    request[KCUPS_PRINTER_NAME] = fromPrinterName;
-    request[KCUPS_JOB_ID] = jobId;
-    request[KCUPS_JOB_PRINTER_URI] = toPrinterName;
+    KIppRequest request(CUPS_MOVE_JOB, "/jobs/");
+    request.addPrinterUri(fromPrinterName);
+    request.addInteger(IPP_TAG_OPERATION, IPP_TAG_INTEGER, KCUPS_JOB_ID, jobId);
+    request.addString(IPP_TAG_OPERATION, IPP_TAG_URI, KCUPS_JOB_PRINTER_URI, toPrinterName);
 
-    doOperation(CUPS_MOVE_JOB, QLatin1String("/jobs/"), request);
+    process(request);
 }
 
 void KCupsRequest::invokeMethod(const char *method,
@@ -601,18 +580,15 @@ void KCupsRequest::invokeMethod(const char *method,
     }
 }
 
-void KCupsRequest::doOperation(int operation, const QString &resource, const QVariantHash &request)
+void KCupsRequest::process(const KIppRequest &request)
 {
     if (m_connection->readyToStart()) {
-        m_connection->request(static_cast<ipp_op_e>(operation),
-                              resource.toUtf8(),
-                              request,
-                              false);
+        m_connection->request(request);
 
         setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
-        invokeMethod("doOperation", operation, resource, request);
+        invokeMethod("process", qVariantFromValue(request));
     }
 }
 
